from decimal import Decimal, InvalidOperation

from django.core.exceptions import ValidationError
from django.core import validators
from django.db import models
from django import forms
from django.utils.translation import ugettext_lazy as _

class PercentagePerYear(list):
    def __init__(self, sequence):
        super(PercentagePerYear, self).__init__(sequence)

    def __str__(self):
        return ','.join(map(lambda p: ':'.join(map(str, [p[0], int(100*p[1])])), self))

class EuroField(models.DecimalField):
    def __init__(self, *args, **kwargs):
        kwargs['max_digits'] = 8
        kwargs['decimal_places'] = 2
        super(EuroField, self).__init__(*args, **kwargs)

def assertion_error_to_validation_error(fun):
    def f(*args, **kwargs):
        try:
            return fun(*args, **kwargs)
        except (AssertionError, ValueError), e:
            raise ValidationError(*e.args)
    return f
@assertion_error_to_validation_error
def check_percentage_per_year(value):
    years = [a for a,b in value]
    percentages = [b for a,b in value]
    # ordered
    assert years == sorted(years), 'years are not ordered'
    # sum equals 100
    assert sum(percentages) == 1, 'percentage does not sum to 100'
    # no duplicate year
    assert len(years) == len(set(years)), 'years are not unique'
    # consecutive
    assert years == range(years[0],years[0]+len(years)), 'years are not consecutive'

def parse_percentage_per_year(value):
    msg = _('field must be numeric values separated by commas')
    values = value.split(',')
    decimals = []
    for value in values:
        try:
            year, decimal = value.split(':')
        except ValueError:
            return ValidationError(msg)
        try:
            year = int(year.strip())
        except ValueError:
            return ValidationError(msg)
        try:
            decimal = Decimal(value.strip())
        except InvalidOperation:
            return ValidationError(msg)
        decimals.append(decimal)
    return PercentagePerYear(decimals)

class PercentagePerYearFormField(forms.Field):
    default_error_messages = {
            'invalid': _('field must be numeric values separated by commas'),
    }

    def to_python(self, value):
        if value in validators.EMPTY_VALUES:
            return None
        if isinstance(PercentagePerYear, value):
            return value
        if not isinstance(unicode, value):
            return ValidationError(self.default_error_messages['invalid'])
        return parse_percentage_per_year(value)

class PercentagePerYearField(models.Field):
    default_validators = [ check_percentage_per_year ]
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 64
        return super(PercentagePerYearField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, PercentagePerYear):
            return value
        if value is not None:
            percentage_per_year = []
            pairs = value.split(',')
            for pair in pairs:
                year, percentage = map(value.__class__.strip, pair.split(':'))
                year = int(year)
                percentage = Decimal(percentage)/Decimal(100)
                percentage_per_year.append((year,percentage))
            try:
                return PercentagePerYear(percentage_per_year)
            except forms.ValidationError:
                return value
        return value

    def get_prep_value(self, value):
        if isinstance(value, PercentagePerYear):
            return unicode(value)
        elif value is not None:
            return unicode(parse_percentage_per_year(value))
        return value

    def get_internal_type(self):
        return 'CharField'

    def formfield(self, **kwargs):
        defaults = {'form_class': PercentagePerYearFormField}
        defaults.update(kwargs)
        return super(PercentagePerYearField, self).formfield(**kwargs)

try:
    # Let South handle our custom fields
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^eo_gestion\.eo_facture\.fields\."])
except ImportError:
    pass
