# -*- coding: utf-8 -*-
import datetime as dt

from django.contrib import admin
from django.conf.urls import patterns, url
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.contrib.admin.options import BaseModelAdmin
import django.http as http
from django.contrib.humanize.templatetags.humanize import ordinal
from django.forms.models import BaseInlineFormSet

from . import forms, models, views
import eo_gestion.admin

class LookupAllowed(object):
    def lookup_allowed(self, *args, **kwargs):
        return True

class SelectRelatedMixin(object):
    def get_queryset(self, request):
        qs = super(SelectRelatedMixin, self).get_queryset(request)
        return qs.select_related()

class CommonPaymentInline(BaseModelAdmin):
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'facture' and request.path.endswith('/add/'):
            kwargs['queryset'] = models.Facture.objects.avec_solde()
        if db_field.name == 'ligne_banque_pop' and request.path.endswith('/add/'):
            kwargs['queryset'] = models.encaissements_avec_solde_non_affecte()
        return super(CommonPaymentInline, self).formfield_for_foreignkey(db_field,
                request, **kwargs)

class PrestationInline(SelectRelatedMixin, admin.TabularInline):
    model = models.Prestation
    form = forms.PrestationForm

class FactureInline(SelectRelatedMixin, admin.TabularInline):
    model = models.Facture
    form = forms.FactureForm

class PaymentInlineFormset(BaseInlineFormSet):
    def __init__(self, *args, **kwargs):
        super(PaymentInlineFormset, self).__init__(*args, **kwargs)
        for form in self.forms:
            if form.instance.id is None:
                field = form.fields['ligne_banque_pop']
                if not hasattr(field, 'parent_instance'):
                    field.queryset = models.encaissements_avec_solde_non_affecte()
                field = form.fields['facture']
                if not hasattr(field, 'parent_instance'):
                    field.queryset = models.Facture.objects.avec_solde()


class PaymentInline(SelectRelatedMixin, admin.TabularInline, CommonPaymentInline):
    form = forms.PaymentForm

    model = models.Payment
    list_display = [ 'facture', 'ligne_banque_pop', 'montant_affecte' ]
    formset = PaymentInlineFormset
    extra = 0


class LigneInline(SelectRelatedMixin, admin.TabularInline):
    form = forms.LigneForm
    model = models.Ligne
    show_url = True
    original = False
    verbose_name_plural = u"Lignes de facture (vous pouvez les réordonner par drag&drop)"
    extra = 0


class ClientAdmin(admin.ModelAdmin):
    form = forms.ClientForm
    list_display = [ 'nom', 'adresse', 'email', 'telephone' ]
    list_editable = [ 'email', 'telephone' ]
    ordering = [ 'nom' ]
    search_fields = [ 'nom', 'email' ]
    save_on_top = True
    list_select_related = True

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.creator = request.user
        obj.save()

    class Media:
        css = { 'all': ('css/client.css',) }

def show_client(obj):
    url = reverse('admin:eo_facture_client_change', args=[obj.client.id])
    return u'<a href="%s">%s</a>' % (url, obj.client)
show_client.short_description = u'Client'
show_client.admin_order_field = 'client'
show_client.allow_tags = True

class ContratAdmin(LookupAllowed,admin.ModelAdmin):
    form = forms.ContratForm
    inlines = [ PrestationInline, ]
    list_display = [ '__unicode__', show_client, 'montant', 'pourcentage_facture', 'creation', 'creator' ]
    list_filter = [ 'client' ]
    list_select_related = True
    save_on_top = True
    search_fields = [ 'intitule', 'client__nom' ]
    readonly_fields = [ 'creator' ]
    list_select_related = True

    def get_queryset(self, request):
        qs = super(ContratAdmin, self).get_queryset(request)
        qs = qs.prefetch_related('prestations')
        qs = qs.prefetch_related('factures__lignes')
        return qs

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.creator = request.user
        obj.save()

    class Media:
        css = { 'all': ('css/contrat.css',) }

    def duplicate(self, request, contract):
        contrat = models.Contrat.objects.get(id=contract)
        if request.method == 'POST':
            form = forms.DuplicateContractForm(request=request,
                    data=request.POST)
            if form.is_valid():
               return http.HttpResponseRedirect(reverse('admin:eo_facture_contrat_change',
                   args=[form.cleaned_data['new_contrat'].id]))
        else:
            new_intitule = contrat.intitule
            new_intitule += u' dupliqué le %s' % dt.datetime.now()
            form = forms.DuplicateContractForm(initial={'contrat':contrat.id,
                'new_intitule':new_intitule} )
        context = {
                'form': form,
                'contrat': contrat,
                'app_label': self.model._meta.app_label,
                'has_change_permission': self.has_change_permission(request),
                'opts': self.model._meta }

        return render_to_response('admin/eo_facture/contrat/duplicate.html',
                context, RequestContext(request))

    def get_urls(self):
        urls = super(ContratAdmin, self).get_urls()
        duplicate_view = self.admin_site.admin_view(self.duplicate)
        my_urls = patterns('',
            url(r'^(.+)/duplicate/$', duplicate_view,
                name='eo_facture_contrat_duplicate')
        )
        return my_urls + urls


def index(facture):
    return ordinal(facture.index())
index.short_description = "Ordre"
index.allow_tags = True

class FactureAdmin(LookupAllowed,admin.ModelAdmin):
    form = forms.FactureForm
    inlines = [ LigneInline, PaymentInline ]
    list_display = [ 'code', index, 'show_contrat', 'show_client', 'emission', 'creator', 'montant', 'montant_ttc', 'solde', 'paid' ]
    list_filter = [ 'paid', 'emission', 'creator', 'client' ]
    fields = [ 'proforma', 'ordre', 'client', 'contrat', 'intitule', 'notes', 'taux_tva', 'emission', 'echeance', 'paid', 'solde', 'montant', 'montant_ttc', 'account_on_previous_period' ]
    readonly_fields = [ 'creator', 'solde', 'ordre', 'montant', 'montant_ttc' ]
    date_hierarchy = 'emission'
    list_select_related = True
    save_on_top = True
    search_fields = [ 'contrat__intitule', 'contrat__client__nom' ]
    list_select_related = True

    def get_queryset(self, request):
        from django.db import connection
        qs = super(FactureAdmin, self).queryset(request)
        qs = qs.prefetch_related('lignes__facture__client')
        qs = qs.prefetch_related('payments__ligne_banque_pop')
        qs = qs.prefetch_related('contrat__factures')
        if connection.vendor == 'postgresql':
            qs = qs.extra({'year': 'EXTRACT(year FROM emission)'})
        elif connection.vendor == 'sqlite':
            qs = qs.extra({'year': "strftime('%Y', emission)"})
        else:
            qs = qs.extra({'year': 'YEAR(emission)'})
        qs = qs.order_by('-year', '-proforma', '-ordre')
        return qs

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.creator = request.user
        obj.save()

    class Media:
        js = ('js/jquery.js', 'js/jquery-ui.js', 'js/menu-sort.js')

    def add_simple(self, request):
        context = {}
        if request.method == 'POST':
            form = forms.RapidFactureForm(request=request, data=request.POST)
            if form.is_valid():
               return http.HttpResponseRedirect(reverse('admin:eo_facture_facture_change',
                   args=[form.cleaned_data['facture'].id]))
        else:
            form = forms.RapidFactureForm(initial=request.REQUEST)
        context = {
                'form': form,
                'app_label': self.model._meta.app_label,
                'has_change_permission': self.has_change_permission(request),
                'opts': self.model._meta }

        return render_to_response('admin/eo_facture/facture/add_simple.html',
                context, RequestContext(request))

    def get_urls(self):
        urls = super(FactureAdmin, self).get_urls()
        my_urls = patterns('',
            (r'^view/([^/]*)', views.facture),
            url(r'^add_simple/$', self.admin_site.admin_view(self.add_simple),
                name='eo_facture_facture_add_simple'),
            url(r'^(.+)/view_pdf/$', views.facture_pdf,
                name='eo_facture_facture_print'),
            url(r'^(.+)/view/$', views.facture,
                name='eo_facture_facture_html')
        )
        return my_urls + urls

    def show_client(self, obj):
        url = reverse('admin:eo_facture_client_change', args=[obj.client.id])
        return u'<a href="%s">%s</a>' % (url, obj.client)
    show_client.short_description = u'Client'
    show_client.allow_tags = True

    def show_contrat(self, obj):
        if obj.contrat:
            url = reverse('admin:eo_facture_contrat_change', args=[obj.contrat.id])
            return u'<a href="%s">%s</a>' % (url, obj.contrat)
        return None
    show_contrat.short_description = u'Contrat'
    show_contrat.allow_tags = True


class PaymentAdmin(LookupAllowed, admin.ModelAdmin, CommonPaymentInline):
    form = forms.PaymentForm
    list_display = [ 'facture', 'ligne_banque_pop', 'montant_affecte' ]
    list_filter = [ 'ligne_banque_pop__date_valeur', 'facture', 'facture__contrat', 'facture__contrat__client' ]
    list_select_related = True

class PrestationAdmin(admin.ModelAdmin):
    form = forms.PrestationForm
    list_select_related = True

admin.site.register(models.Client, ClientAdmin)
admin.site.register(models.Contrat, ContratAdmin)
admin.site.register(models.Facture, FactureAdmin)
admin.site.register(models.Prestation, PrestationAdmin)

eo_gestion.admin.site.register(models.Client, ClientAdmin)
eo_gestion.admin.site.register(models.Contrat, ContratAdmin)
eo_gestion.admin.site.register(models.Facture, FactureAdmin)
eo_gestion.admin.site.register(models.Payment, PaymentAdmin)
