# -*- encoding: utf-8 -*-
from datetime import date, timedelta, datetime
from decimal import Decimal, InvalidOperation
from collections import defaultdict

from django import template
from django.utils.timesince import timesince
from django.core.urlresolvers import reverse

from eo_gestion.eo_facture.models import Contrat, Facture, DELAI_PAIEMENT
from eo_gestion.eo_banque.models import LigneBanquePop
from eo_gestion.utils import percentage

register = template.Library()

@register.inclusion_tag('eo_facture/impayee.html')
def impayees():
    factures_impayees = [f for f in Facture.objects \
            .select_related('client', 'contrat', 'contrat__client') \
            .prefetch_related('lignes', 'payments') \
            .avec_solde() \
            .filter(paid=False).order_by('emission') if f.solde() < 0]
    factures = []
    soldes = [ -f.solde() for f in factures_impayees ]
    soldes_ht = [ f.montant * (solde/ f.montant_ttc) if f.montant_ttc else Decimal(0) for solde, f in zip(soldes, factures_impayees) ]
    solde = sum(soldes)
    solde_ht = sum(soldes_ht)
    today = date.today()
    after = datetime.now() - timedelta(days=365)
    after = after.date()
    lignes = LigneBanquePop.objects.filter(
                    date_operation__gte=after,
                    montant__in=soldes, payments=None)
    for f in factures_impayees:
        vieille = False
        if (today - f.emission) > timedelta(days=DELAI_PAIEMENT):
            vieille = True
        encaissements = [ligne for ligne in lignes if ligne.montant == f.montant_ttc]
        factures.append({
            'facture': f,
            'encaissements': encaissements,
            'vieille': vieille
        })
    return { 'montant': solde, 'montant_ht': solde_ht,
             'factures': factures  }

@register.inclusion_tag('eo_facture/income.html')
def income():
    def AmountByYear():
        return defaultdict(lambda:Decimal(0))

    invoices = Facture.objects.filter(proforma=False) \
            .select_related('client', 'contrat', 'contrat__client') \
            .prefetch_related('lignes', 'payments')
    paid_by_year = AmountByYear()
    today_year = date.today().year
    sous_traite_by_year = AmountByYear()
    for invoice in invoices:
        if invoice.paid:
            paid_by_year[invoice.accounting_year] += invoice.montant
        else:
            tva = invoice.tva
            ht = invoice.montant
            try:
                taux_moyen = tva / ht
            except (ZeroDivisionError, InvalidOperation):
                taux_moyen = Decimal(0)
            paid_by_year[invoice.accounting_year] += (invoice.montant_ttc + invoice.solde()) * Decimal(1) / ( Decimal(1) + taux_moyen )
    invoiced_by_year = defaultdict(lambda:Decimal(0))
    invoiced_by_year_and_client = defaultdict(lambda:defaultdict(lambda:Decimal(0)))
    for invoice in invoices:
        montant = invoice.montant - invoice.sous_traite
        invoiced_by_year[invoice.accounting_year] += montant
        sous_traite_by_year[invoice.accounting_year] += invoice.sous_traite
        invoiced_by_year_and_client[invoice.accounting_year][invoice.client.id] += montant
    invoiced_clients_by_year = {}
    for year in invoiced_by_year_and_client:
        clients = invoiced_by_year_and_client[year].keys()
        clients.sort(key=lambda x: -invoiced_by_year_and_client[year][x])
        invoiced_clients_by_year[year] = clients
    contracted_by_year = defaultdict(lambda:Decimal(0))
    for contract in Contrat.objects.all().prefetch_related('factures', 'factures__lignes', 'prestations'):
        adjust = 0
        for year, fraction in contract.percentage_per_year:
            amount = contract.montant()*fraction
            if year > today_year:
                contracted_by_year[year] += amount
            else:
                adjust += amount
        for f in contract.factures.all():
            if f.proforma:
                continue
            if f.accounting_year <= today_year:
               adjust -= f.montant - f.sous_traite
        if adjust > 0:
            contracted_by_year[today_year] += adjust
    this_year = date.today().year
    income_by_year = []
    for year in invoiced_by_year:
        invoiced_by_year[year] -= paid_by_year[year]
    for year in range(this_year-1,this_year+2):
        income_by_year.append({
            'year': year,
            'paid': paid_by_year[year],
            'invoiced': invoiced_by_year[year],
            'contracted': contracted_by_year[year],
            'sous_traite': sous_traite_by_year[year],
            'total': paid_by_year[year]+invoiced_by_year[year]+contracted_by_year[year]})
    return { 'income_by_year': income_by_year }

def zero_dict():
    return defaultdict(lambda:Decimal(0))

class StringWithHref(unicode):
    def __new__(cls, v, href=None):
        return unicode.__new__(cls, v)

    def __init__(self, v, href=None):
        super(StringWithHref, self).__init__(v)
        self.href = href

def client_and_link(c):
    s = unicode(c)
    url = reverse('admin:eo_facture_client_change', args=(c.id,))
    return StringWithHref(s, url)

def dict_of_list():
    return defaultdict(lambda:[])

@register.inclusion_tag('eo_facture/table.html')
def income_by_clients(year=None):
    if not year:
        year = date.today().year
    contracted_by_clients = zero_dict()
    invoiced_by_clients = zero_dict()
    total_by_clients = zero_dict()
    contracts_by_clients = dict_of_list()
    invoices_by_clients = dict_of_list()
    for contrat in Contrat.objects \
            .select_related('client') \
            .prefetch_related('factures', 'factures__lignes', 'prestations'):
        # how much as already been invoiced before previsions
        adjust = 0
        invoiced = 0
        for percent_year, fraction in contrat.percentage_per_year:
            if percent_year > year:
                continue
            adjust += contrat.montant()*fraction
        for facture in contrat.factures.all():
            if facture.proforma:
                continue
            if facture.accounting_year <= year:
                adjust -= facture.montant
            if facture.accounting_year == year:
                invoiced += facture.montant
        if adjust > 0:
            contracted_by_clients[contrat.client] += adjust
            total_by_clients[contrat.client] += adjust
        if invoiced:
            invoiced_by_clients[contrat.client] += invoiced
            total_by_clients[contrat.client] += invoiced
    for facture in Facture.objects \
            .select_related('client') \
            .filter(contrat__isnull=True) \
            .for_year(year) \
            .prefetch_related('lignes'):
        if facture.proforma:
            continue
        invoiced_by_clients[facture.client] += facture.montant
        total_by_clients[facture.client] += facture.montant
    total = sum(total_by_clients.values())
    total_invoiced = sum(invoiced_by_clients.values())
    total_contracted = sum(contracted_by_clients.values())
    percent_by_clients = dict([ (i, Decimal(100)*v/total) for i, v in total_by_clients.iteritems()])
    clients = total_by_clients.keys()
    clients.sort(key=lambda x: -total_by_clients[x])
    running_total = Decimal(0)
    # compute pareto index
    pareto = dict()
    for client in clients:
        running_total += percent_by_clients[client]
        pareto[client] = running_total
    return dict(title=u"Chiffre d'affaire prévisionnel par client pour %s" % year,
            name="previsional-income-by-client-%s" % year,
            headers=[('client', 'Client'),
                     ('income', "Chiffre d'affaire"),
                     ('invoiced', u"Facturé"),
                     ('percent_invoiced', u"Pourcentage facturé"),
                     ('contracted', u"Contracté"),
                     ('percent_contracted', u"Pourcentage contracté"),
                     ('percent', 'Pourcentage'),
                     ('pareto', 'Pareto')],
            contracts_by_clients=contracts_by_clients.iteritems(),
            invoices_by_clients=invoices_by_clients.iteritems(),
            table=[(client_and_link(c),
                   total_by_clients[c],
                   invoiced_by_clients[c],
                   percentage(invoiced_by_clients[c], total_by_clients[c]),
                   contracted_by_clients[c],
                   percentage(contracted_by_clients[c], total_by_clients[c]),
                   percent_by_clients[c], pareto[c])
                    for c in clients]+[('Total', total, total_invoiced, percentage(total_invoiced, total), total_contracted, percentage(total_contracted, total), Decimal(100), '')])

@register.inclusion_tag('eo_facture/a_facturer.html')
def a_facturer():
    l = []
    for contrat in Contrat.objects.all() \
            .select_related('client') \
            .prefetch_related('prestations', 'factures', 'factures__lignes'):
        facture = contrat.montant_facture()
        a_facture = contrat.montant()
        if facture < a_facture:
            if contrat.factures.count() > 0:
                depuis = max(facture.emission for facture in contrat.factures.all())
            else:
                depuis = contrat.creation
            depuis = (date.today()-depuis).days
            l.append({ 'contrat': contrat,
                'pourcentage': (facture/a_facture)*Decimal(100),
                'montant': a_facture-facture,
                'depuis': depuis })
    l.sort(key=lambda x: -x['depuis'])
    montant = sum([x['montant'] for x in l])
    return { 'a_facturer': l, 'montant': montant }

@register.filter(name='ago')
def ago(date):
    ago = timesince(date)
    # selects only the first part of the returned string
    return ago.split(",")[0]
